@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2018 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -35,22 +35,10 @@
 #include "hkl-gui.h"
 #include "hkl-gui-macros.h"
 #if HKL3D
-# include <gtk/gtkgl.h>
 # include "hkl-gui-3d.h"
 #endif
 #include "hkl-gui-pseudoaxes.h"
 
-#define HKL_GUI_TYPE_WINDOW (hkl_gui_window_get_type ())
-#define HKL_GUI_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HKL_GUI_TYPE_WINDOW, HklGuiWindow))
-#define HKL_GUI_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), HKL_GUI_TYPE_WINDOW, HklGuiWindowClass))
-#define HKL_GUI_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HKL_GUI_TYPE_WINDOW))
-#define HKL_GUI_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), HKL_GUI_TYPE_WINDOW))
-#define HKL_GUI_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), HKL_GUI_TYPE_WINDOW, HklGuiWindowClass))
-
-#define EMBED_BREAKPOINT  asm volatile ("int3;")
-
-G_DEFINE_TYPE (HklGuiWindow, hkl_gui_window, G_TYPE_OBJECT);
-
 typedef enum  {
 	REFLECTION_COL_INDEX = 0,
 	REFLECTION_COL_H,
@@ -208,6 +196,8 @@ diffractometer_set_solution(struct diffractometer_t *self,
 /* HklGuiWindow */
 /****************/
 
+typedef struct _HklGuiWindowPrivate HklGuiWindowPrivate;
+
 struct _HklGuiWindowPrivate {
 	GtkBuilder* builder;
 	GtkLabel* label_UB11;
@@ -281,10 +271,9 @@ struct _HklGuiWindowPrivate {
 	GtkToolButton* toolbutton_del_crystal;
 	GtkToolButton* toolbutton_affiner;
 	GtkStatusbar* statusbar;
-	GtkImageMenuItem* menuitem5;
-	GtkVBox* box_info_bar; /* fake for the infor bar */
-	GtkVBox* vbox7;
-	GtkVBox* vbox2;
+	GtkMenuItem* menuitem5;
+	GtkBox* vbox7;
+	GtkBox* vbox2;
 	GtkDialog* dialog1;
 	GtkButton* button1;
 	GtkComboBox* combobox1;
@@ -308,6 +297,15 @@ struct _HklGuiWindowPrivate {
 	HklLattice *reciprocal;
 };
 
+
+struct _HklGuiWindow {
+	GObject parent_instance;
+	HklGuiWindowPrivate * priv;
+};
+
+
+G_DEFINE_TYPE_WITH_PRIVATE (HklGuiWindow, hkl_gui_window, G_TYPE_OBJECT);
+
 #define HKL_GUI_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), HKL_GUI_TYPE_WINDOW, HklGuiWindowPrivate))
 
 static gboolean
@@ -470,11 +468,13 @@ hkl_gui_window_get_widgets_and_objects_from_ui (HklGuiWindow* self)
 
 	get_object(builder, GTK_STATUSBAR, priv, statusbar);
 
-	get_object(builder, GTK_IMAGE_MENU_ITEM, priv, menuitem5);
+	get_object(builder, GTK_MENU_ITEM, priv, menuitem5);
 
-	get_object(builder, GTK_VBOX, priv, vbox7);
-	get_object(builder, GTK_VBOX, priv, vbox2);
-	get_object(builder, GTK_VBOX, priv, box_info_bar);
+	get_object(builder, GTK_BOX, priv, vbox7);
+	get_object(builder, GTK_BOX, priv, vbox2);
+
+	get_object(builder, GTK_INFO_BAR, priv, info_bar);
+	get_object(builder, GTK_LABEL, priv, info_message);
 
 	get_object(builder, GTK_DIALOG, priv, dialog1);
 
@@ -921,38 +921,6 @@ set_up_tree_view_solutions (HklGuiWindow* self)
 	update_solutions (self);
 }
 
-void
-set_up_info_bar(HklGuiWindow *self)
-{
-	HklGuiWindowPrivate *priv = HKL_GUI_WINDOW_GET_PRIVATE(self);
-	GtkWidget *content_area;
-
-	g_return_if_fail (self != NULL);
-
-	/* set up info bar until we can use glade for this purpose or
-	 * switch to gtk3 */
-	if (priv->info_bar)
-		return;
-
-	priv->info_bar = GTK_INFO_BAR(gtk_info_bar_new ());
-	gtk_widget_set_no_show_all (GTK_WIDGET(priv->info_bar), TRUE);
-
-	priv->info_message = GTK_LABEL(gtk_label_new (""));
-	gtk_widget_show (GTK_WIDGET(priv->info_message));
-
-	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->info_bar));
-	gtk_container_add (GTK_CONTAINER (content_area),
-			   GTK_WIDGET(priv->info_message));
-	gtk_info_bar_add_button (priv->info_bar,
-				 GTK_STOCK_OK, GTK_RESPONSE_OK);
-	g_signal_connect (priv->info_bar, "response",
-			  G_CALLBACK (gtk_widget_hide), NULL);
-
-	gtk_box_pack_start(GTK_BOX(priv->box_info_bar),
-			   GTK_WIDGET(priv->info_bar),
-			   TRUE, TRUE, 0);
-}
-
 static void
 set_up_lambda(HklGuiWindow *self)
 {
@@ -1035,7 +1003,6 @@ hkl_gui_window_combobox1_changed_cb(GtkComboBox *combobox, gpointer *user_data)
 		set_up_tree_view_axes(self);
 		set_up_tree_view_pseudo_axes(self);
 		set_up_tree_view_solutions(self);
-		set_up_info_bar(self);
 		set_up_3D(self);
 	}
 }
@@ -1295,19 +1262,20 @@ hkl_gui_window_treeview_solutions_cursor_changed_cb (GtkTreeView *tree_view,
 	const HklGeometryListItem *solution;
 
 	gtk_tree_view_get_cursor (tree_view, &path, &focus_column);
-	gtk_tree_model_get_iter (GTK_TREE_MODEL(priv->liststore_solutions), &iter, path);
-	gtk_tree_model_get (GTK_TREE_MODEL(priv->liststore_solutions), &iter,
-			    SOLUTION_COL_HKL_GEOMETRY_LIST_ITEM, &solution,
-			    -1);
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL(priv->liststore_solutions), &iter, path)) {
+		gtk_tree_model_get (GTK_TREE_MODEL(priv->liststore_solutions), &iter,
+				    SOLUTION_COL_HKL_GEOMETRY_LIST_ITEM, &solution,
+				    -1);
 
-	diffractometer_set_solution(priv->diffractometer, solution);
+		diffractometer_set_solution(priv->diffractometer, solution);
 
-	update_axes (self);
-	update_pseudo_axes (self);
-	update_pseudo_axes_frames (self);
-	update_3d(self);
+		update_axes (self);
+		update_pseudo_axes (self);
+		update_pseudo_axes_frames (self);
+		update_3d(self);
 
-	gtk_tree_path_free (path);
+		gtk_tree_path_free (path);
+	}
 }
 
 /* reflection h k l */
@@ -2330,8 +2298,6 @@ hkl_gui_window_class_init (HklGuiWindowClass *class)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (class);
 
-	g_type_class_add_private (class, sizeof (HklGuiWindowPrivate));
-
 	/* virtual method */
 	gobject_class->finalize = finalize;
 }
@@ -2360,9 +2326,7 @@ static void hkl_gui_window_init (HklGuiWindow * self)
 int main (int argc, char ** argv)
 {
 	gtk_init (&argc, &argv);
-#ifdef HKL3D
-	gtk_gl_init(&argc, &argv);
-#endif
+
 	hkl_gui_window_new ();
 
 	gtk_main ();
