@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2018 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -61,6 +61,7 @@ static inline GQuark hkl_parameter_error_quark (void)
 
 typedef enum {
 	HKL_PARAMETER_ERROR_MIN_MAX_SET, /* can not set the min max */
+	HKL_PARAMETER_ERROR_VALUE_SET, /* can not set the value */
 } HklParameterError;
 
 /****************/
@@ -140,6 +141,15 @@ static inline int hkl_parameter_value_set_real(HklParameter *self, double value,
 {
 	hkl_error (error == NULL || *error == NULL);
 
+	if (!isfinite(value)){
+		g_set_error(error,
+			    HKL_PARAMETER_ERROR,
+			    HKL_PARAMETER_ERROR_VALUE_SET,
+			    "It is forbiden to set the (%s) parameter value with the non-finite %f value\n",
+			    self->name, value);
+		return FALSE;
+	}
+
 	switch (unit_type) {
 	case HKL_UNIT_DEFAULT:
 		self->_value = value;
@@ -254,6 +264,7 @@ extern HklVector hkl_parameter_transformation_apply(const HklParameter *self,
 /********************/
 
 typedef darray(HklParameter *) darray_parameter;
+typedef darray(const HklParameter) darray_const_parameter;
 
 G_END_DECLS
 
