@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2018 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -42,6 +42,8 @@ typedef struct _HklModeInfo HklModeInfo;
 typedef struct _HklMode HklMode;
 typedef struct _HklEngineInfo HklEngineInfo;
 typedef struct _HklEngineOperations HklEngineOperations;
+typedef struct _HklEngineListInfo HklEngineListInfo;
+typedef struct _HklEngineListOperations HklEngineListOperations;
 
 typedef darray(HklMode *) darray_mode;
 
@@ -272,17 +274,36 @@ struct _HklEngine
 };
 
 
+struct _HklEngineListInfo {
+	const darray(const HklParameter *) parameters;
+};
+
+#define HKL_ENGINE_LIST_INFO_DEFAULTS .parameters = darray_new()
+#define HKL_ENGINE_LIST_INFO(_parameters) .parameters = DARRAY(_parameters)
+
+struct _HklEngineListOperations
+{
+	void (* free)(HklEngineList *self);
+	int (* post_engine_set)(HklEngineList *self);
+};
+
+#define HKL_ENGINE_LIST_OPERATIONS_DEFAULTS .free=hkl_engine_list_free_real, \
+		.post_engine_set=hkl_engine_list_post_engine_set_real
+
 struct _HklEngineList
 {
-	_darray(HklEngine *);
+	_darray(HklEngine *); /* must be the first memeber */
+	const HklEngineListInfo *info;
+	const HklEngineListOperations *ops;
 	HklGeometryList *geometries;
 	HklGeometry *geometry;
 	HklDetector *detector;
 	HklSample *sample;
 	darray_parameter pseudo_axes;
+	darray_parameter parameters;
+	darray_string parameters_names;
 };
 
-
 #define HKL_ENGINE_ERROR hkl_engine_error_quark ()
 
 
@@ -543,6 +564,19 @@ static inline int hkl_engine_get(HklEngine *self, GError **error)
 }
 
 
+/**
+ * hkl_engine_list_post_process_free: (skip)
+ * @self: the #HklEngineList to modify
+ *
+ * apply a post set method.
+ **/
+static int hkl_engine_list_post_engine_set(HklEngineList *self)
+{
+	return self->ops->post_engine_set(self);
+}
+
+
+
 /**
  * hkl_engine_set: (skip)
  * @self: the HklEngine
@@ -578,6 +612,7 @@ static inline int hkl_engine_set(HklEngine *self, GError **error)
 	hkl_assert(error == NULL || *error == NULL);
 
 	hkl_geometry_list_multiply(self->engines->geometries);
+	hkl_engine_list_post_engine_set(self->engines);
 	hkl_geometry_list_multiply_from_range(self->engines->geometries);
 	hkl_geometry_list_remove_invalid(self->engines->geometries);
 	hkl_geometry_list_sort(self->engines->geometries, self->engines->geometry);
@@ -608,49 +643,10 @@ static inline GQuark hkl_engine_list_error_quark (void)
 typedef enum {
 	HKL_ENGINE_LIST_ERROR_ENGINE_GET_BY_NAME, /* can not set this geometry */
 	HKL_ENGINE_LIST_ERROR_PSEUDO_AXIS_GET_BY_NAME, /* can not set this geometry */
+	HKL_ENGINE_LIST_ERROR_PARAMETER_GET, /* can not get the parameter */
+	HKL_ENGINE_LIST_ERROR_PARAMETER_SET, /* can not set the parameter */
 } HklEngineListError;
 
-
-/**
- * hkl_engine_list_new: (skip)
- *
- * default constructor
- *
- * Returns:
- **/
-static inline HklEngineList *hkl_engine_list_new(void)
-{
-	HklEngineList *self = NULL;
-
-	self = HKL_MALLOC(HklEngineList);
-
-	darray_init(*self);
-
-	self->geometries = hkl_geometry_list_new();
-
-	self->geometry = NULL;
-	self->detector = NULL;
-	self->sample = NULL;
-
-	darray_init(self->pseudo_axes);
-
-	return self;
-}
-
-
-/**
- * hkl_engine_list_new_copy: (skip)
- * @self:
- *
- * dummy copy constructor for the binding
- *
- * Returns: (transfer none): NULL all the time the structure is non-copyable
- **/
-static inline const HklEngineList *hkl_engine_list_new_copy(UNUSED const HklEngineList *self)
-{
-	return NULL;
-}
-
 /**
  * hkl_engine_list_clear: (skip)
  * @self: the engine list to clear
@@ -671,8 +667,107 @@ static inline void hkl_engine_list_clear(HklEngineList *self)
 		hkl_parameter_free(*parameter);
 	}
 	darray_free(self->pseudo_axes);
+
+	darray_foreach(parameter, self->parameters){
+		hkl_parameter_free(*parameter);
+	}
+	darray_free(self->parameters);
+	darray_free(self->parameters_names);
 }
 
+/**
+ * hkl_engine_list_free_real: (skip)
+ * @self: the #HklEngineList to destroy
+ *
+ * destructor
+ **/
+static inline void hkl_engine_list_free_real(HklEngineList *self)
+{
+	hkl_engine_list_clear(self);
+	hkl_geometry_list_free(self->geometries);
+	free(self);
+}
+
+/**
+ * hkl_engine_list_post_process_real: (skip)
+ * @self: the #HklEngineList to destroy
+ *
+ * destructor
+ **/
+static inline int hkl_engine_list_post_engine_set_real(HklEngineList *self)
+{
+	return FALSE;
+}
+
+/**
+ * hkl_engine_list_new_with_info: (skip)
+ * @info: the info part of the HklEngineList
+ *
+ * default constructor with info part
+ *
+ * Returns:
+ **/
+static inline HklEngineList *hkl_engine_list_new_with_info(const HklEngineListInfo *info,
+							   const HklEngineListOperations *ops)
+{
+	HklEngineList *self = NULL;
+	const HklParameter **parameter;
+
+	self = HKL_MALLOC(HklEngineList);
+
+	/* This code needs _darray to be at start of HklEngineList */
+	BUILD_ASSERT(offsetof(HklEngineList, item) == 0);
+	darray_init(*self);
+
+	self->info = info;
+	self->ops = ops;
+
+	self->geometries = hkl_geometry_list_new();
+
+	self->geometry = NULL;
+	self->detector = NULL;
+	self->sample = NULL;
+
+	darray_init(self->pseudo_axes);
+
+	darray_init(self->parameters);
+	darray_init(self->parameters_names);
+	darray_foreach(parameter, info->parameters){
+		darray_append(self->parameters, hkl_parameter_new_copy(*parameter));
+		darray_append(self->parameters_names, (*parameter)->name);
+	};
+
+	return self;
+}
+
+/**
+ * hkl_engine_list_new: (skip)
+ *
+ * default constructor
+ *
+ * Returns:
+ **/
+static inline HklEngineList *hkl_engine_list_new(void)
+{
+	static const HklEngineListInfo info = {HKL_ENGINE_LIST_INFO_DEFAULTS};
+	static const HklEngineListOperations ops = {HKL_ENGINE_LIST_OPERATIONS_DEFAULTS};
+
+	return hkl_engine_list_new_with_info(&info, &ops);
+}
+
+
+/**
+ * hkl_engine_list_new_copy: (skip)
+ * @self:
+ *
+ * dummy copy constructor for the binding
+ *
+ * Returns: (transfer none): NULL all the time the structure is non-copyable
+ **/
+static inline const HklEngineList *hkl_engine_list_new_copy(UNUSED const HklEngineList *self)
+{
+	return NULL;
+}
 
 G_END_DECLS
 
